@@ -855,6 +855,8 @@ _rr_types={
 	"APL"  : ldns.LDNS_RR_TYPE_APL,
 	"ATMA" : ldns.LDNS_RR_TYPE_ATMA,
 	"AXFR" : ldns.LDNS_RR_TYPE_AXFR,
+	"CDNSKEY" : ldns.LDNS_RR_TYPE_CDNSKEY,
+	"CDS" : ldns.LDNS_RR_TYPE_CDS,
 	"CERT" : ldns.LDNS_RR_TYPE_CERT,
 	"CNAME": ldns.LDNS_RR_TYPE_CNAME,
 	"COUNT": ldns.LDNS_RR_TYPE_COUNT,
@@ -895,6 +897,7 @@ _rr_types={
 	"NSEC3PARAMS" : ldns.LDNS_RR_TYPE_NSEC3PARAMS,
 	"NULL" : ldns.LDNS_RR_TYPE_NULL,
 	"NXT"  : ldns.LDNS_RR_TYPE_NXT,
+	"OPENPGPKEY" : ldns.LDNS_RR_TYPE_OPENPGPKEY,
 	"OPT"  : ldns.LDNS_RR_TYPE_OPT,
 	"PTR"  : ldns.LDNS_RR_TYPE_PTR,
 	"PX"   : ldns.LDNS_RR_TYPE_PX,
@@ -906,6 +909,7 @@ _rr_types={
 	"SOA"  : ldns.LDNS_RR_TYPE_SOA,
 	"SRV"  : ldns.LDNS_RR_TYPE_SRV,
 	"SSHFP": ldns.LDNS_RR_TYPE_SSHFP,
+	"TLSA" : ldns.LDNS_RR_TYPE_TLSA,
 	"TSIG" : ldns.LDNS_RR_TYPE_TSIG,
 	"TXT"  : ldns.LDNS_RR_TYPE_TXT,
 	"UID"  : ldns.LDNS_RR_TYPE_UID,
