@@ -30,10 +30,9 @@
 (VAR) = (VAR) & ~(BIT)
 
 extern ldns_rr_list *global_dns_root;
-extern bool qds;
 extern int verbosity;
 
-ldns_pkt *do_trace(ldns_resolver *res,
+void do_trace(ldns_resolver *res,
 			    ldns_rdf *name,
 			    ldns_rr_type type, 
 			    ldns_rr_class c);
@@ -44,8 +43,7 @@ ldns_status do_chase(ldns_resolver *res,
 				 ldns_rr_list *trusted_keys, 
 				 ldns_pkt *pkt_o,
 				 uint16_t qflags,
-				 ldns_rr_list *prev_key_list,
-				 int verbosity);
+				 ldns_rr_list *prev_key_list);
 int do_secure_trace(ldns_resolver *res,
 				ldns_rdf *name,
 				ldns_rr_type type, 
