@@ -182,6 +182,7 @@ main(int argc, char **argv)
 	uint8_t *wire = NULL;
 	size_t wiresize = 0;
 	const char *port = "53";
+	char *keydata;
 
 	srandom(time(NULL) ^ getpid());
 
@@ -203,9 +204,14 @@ main(int argc, char **argv)
                 case 'y':
 			tsig_cred.algorithm = (char*)"hmac-md5.sig-alg.reg.int.";
 			tsig_cred.keyname = optarg;
-			tsig_cred.keydata = strchr(optarg, ':');
-			*tsig_cred.keydata = '\0';
-			tsig_cred.keydata++;
+			keydata = strchr(optarg, ':');
+			if (keydata == NULL) {
+				printf("TSIG argument is not in form "
+					"key:data: %s\n", optarg);
+				exit(1);
+			}
+			*keydata++ = '\0';
+			tsig_cred.keydata = keydata;
 			printf("Sign with %s : %s\n", tsig_cred.keyname,
 				tsig_cred.keydata);
 			break;
@@ -250,7 +256,7 @@ main(int argc, char **argv)
 	ldns_pkt_set_opcode(notify, LDNS_PACKET_NOTIFY);
 	ldns_pkt_push_rr(notify, LDNS_SECTION_QUESTION, question);
 	ldns_pkt_set_aa(notify, true);
-	ldns_pkt_set_id(notify, random()&0xffff);
+	ldns_pkt_set_random_id(notify);
 	if(include_soa) {
 		char buf[10240];
 		ldns_rr *soa_rr=NULL;
@@ -301,7 +307,7 @@ main(int argc, char **argv)
 
 	for(i=0; i<argc; i++)
 	{
-		struct addrinfo hints, *res0, *res;
+		struct addrinfo hints, *res0, *ai_res;
 		int error;
 		int default_family = AF_INET;
 
@@ -317,13 +323,13 @@ main(int argc, char **argv)
 				gai_strerror(error));
 			continue;
 		}
-		for (res = res0; res; res = res->ai_next) {
-			int s = socket(res->ai_family, res->ai_socktype, 
-				res->ai_protocol);
+		for (ai_res = res0; ai_res; ai_res = ai_res->ai_next) {
+			int s = socket(ai_res->ai_family, ai_res->ai_socktype, 
+				ai_res->ai_protocol);
 			if(s == -1)
 				continue;
 			/* send the notify */
-			notify_host(s, res, wire, wiresize, argv[i]);
+			notify_host(s, ai_res, wire, wiresize, argv[i]);
 		}
 		freeaddrinfo(res0);
 	}
