@@ -31,7 +31,7 @@
  *
  * Domain names are stored in \ref ldns_rdf structures, with the type
  * \ref LDNS_RDF_TYPE_DNAME
- * 
+ *
  * This module is *NOT* about the RR type called DNAME.
  */
 
@@ -42,6 +42,10 @@
 #include <ldns/common.h>
 #include <ldns/rdata.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #define LDNS_DNAME_NORMALIZE        tolower
 
 /**
@@ -58,7 +62,7 @@ ldns_rdf *ldns_dname_cat_clone(const ldns_rdf *rd1, const ldns_rdf *rd2);
  * \param[in] rd2 the rightside
  * \return LDNS_STATUS_OK on success
  */
-ldns_status 	ldns_dname_cat(ldns_rdf *rd1, ldns_rdf *rd2);
+ldns_status 	ldns_dname_cat(ldns_rdf *rd1, const ldns_rdf *rd2);
 
 /**
  * Returns a clone of the given dname with the labels
@@ -69,8 +73,21 @@ ldns_status 	ldns_dname_cat(ldns_rdf *rd1, ldns_rdf *rd2);
 ldns_rdf *ldns_dname_reverse(const ldns_rdf *d);
 
 /**
- * chop one label off the left side of a dname. so 
+ * Clones the given dname from the nth label on
+ * \param[in] d The dname to clone
+ * \param[in] n the label nr to clone from, if this is 0, the complete
+ *              dname is cloned
+ * \return A newly allocated *rdf structure, containing the cloned dname,
+ *         or NULL if either d was NULL, not a dname, or if n >=
+ *         label_count
+ */
+ldns_rdf *
+ldns_dname_clone_from(const ldns_rdf *d, uint16_t n);
+
+/**
+ * chop one label off the left side of a dname. so
  * wwww.nlnetlabs.nl, becomes nlnetlabs.nl
+ * This new name is a clone and must be freed with ldns_deep_free()
  * \param[in] d the dname to chop
  * \return the remaining dname
  */
@@ -80,7 +97,7 @@ ldns_rdf *ldns_dname_left_chop(const ldns_rdf *d);
  * count the number of labels inside a LDNS_RDF_DNAME type rdf.
  * \param[in] *r the rdf
  * \return the number of labels
- */     
+ */
 uint8_t  ldns_dname_label_count(const ldns_rdf *r);
 
 /**
@@ -91,9 +108,11 @@ uint8_t  ldns_dname_label_count(const ldns_rdf *r);
 ldns_rdf *ldns_dname_new_frm_str(const char *str);
 
 /**
- * Create a new dname rdf from a string
- * \param[in] s the size of the new dname 
+ * Create a new dname rdf from a string. The data pointer
+ * is stored in the rdf, not a copy of the data
+ * \param[in] s the size of the new dname
  * \param[in] *data pointer to the actual data
+ *
  * \return ldns_rdf*
  */
 ldns_rdf *ldns_dname_new(uint16_t s, void *data);
@@ -102,6 +121,7 @@ ldns_rdf *ldns_dname_new(uint16_t s, void *data);
  * Create a new dname rdf from data (the data is copied)
  * \param[in] size the size of the data
  * \param[in] *data pointer to the actual data
+ *
  * \return ldns_rdf*
  */
 ldns_rdf *ldns_dname_new_frm_data(uint16_t size, const void *data);
@@ -131,10 +151,22 @@ bool ldns_dname_is_subdomain(const ldns_rdf *sub, const ldns_rdf *parent);
  * \return -1 if dname1 comes before dname2, 1 if dname1 comes after dname2, and 0 if they are equal.
  */
 int ldns_dname_compare(const ldns_rdf *dname1, const ldns_rdf *dname2);
+int ldns_dname_compare_v(const void *, const void *);
+
+/**
+ * Checks whether the dname matches the given wildcard
+ * \param[in] dname The dname to check
+ * \param[in] wildcard The wildcard to check with
+ * \return 1 If the wildcard matches, OR if 'wildcard' is not a wildcard and
+ *           the names are *exactly* the same
+ *         0 If the wildcard does not match, or if it is not a wildcard and
+ *           the names are not the same
+ */
+int ldns_dname_match_wildcard(const ldns_rdf *dname, const ldns_rdf *wildcard);
 
 /**
  * check if middle lays in the interval defined by prev and next
- * prev <= middle < next. This is usefull for nsec checking
+ * prev <= middle < next. This is useful for nsec checking
  * \param[in] prev the previous dname
  * \param[in] middle the dname to check
  * \param[in] next the next dname
@@ -149,6 +181,13 @@ int ldns_dname_interval(const ldns_rdf *prev, const ldns_rdf *middle, const ldns
  */
 bool ldns_dname_str_absolute(const char *dname_str);
 
+/**
+ * Checks whether the given dname is absolute (i.e. ends with a '.')
+ * \param[in] *dname a rdf representing the dname
+ * \return true or false
+ */
+bool ldns_dname_absolute(const ldns_rdf *dname);
+
 /**
  * look inside the rdf and if it is an LDNS_RDF_TYPE_DNAME
  * try and retrieve a specific label. The labels are numbered
@@ -159,4 +198,15 @@ bool ldns_dname_str_absolute(const char *dname_str);
  */
 ldns_rdf * ldns_dname_label(const ldns_rdf *rdf, uint8_t labelpos);
 
+/**
+ * Check if dname is a wildcard, starts with *.
+ * \param[in] dname: the rdf to look in
+ * \return true if a wildcard, false if not.
+ */
+int ldns_dname_is_wildcard(const ldns_rdf* dname);
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif	/* LDNS_DNAME_H */
